/*************************************************************************/
/*                                                                       */
/*                IMPROVED SEVEN LEVEL GRAYSCALE FOR HW1                 */
/*                               v 0.1.1                                 */
/*                                                                       */
/*                      by Jonas Lf, 2000-07-21                        */
/*                        f96jolo@dd.chalmers.se                         */
/*                                                                       */
/*    You may use this file in any way you want, but give credit         */
/*    where credit's due.                                                */
/*                                                                       */
/*    This is an example showing how quality seven level grayscale       */
/*    can be acived on the TI89 with 'Hardware Version 1' by changing    */     
/*    the values of the control registers of the LCD. The image is as    */
/*    stable as with standard four level grayscale.                      */
/*                                                                       */
/*************************************************************************/

#include <nostub.h>  // Make NoStub Program
#include <all.h>     // Include All Header Files

int _ti89;           // Produce .89Z File


// Interrupt routine for fliping planes
void grayman(void);
// Index into Table and Table
extern unsigned gp_phase, gp_control[6];
asm ("
grayman:
      movem.l %d0/%a0,-(%sp)
      move.w gp_phase(%pc),%d0
      lea gp_control(%pc),%a0
      move.w 0(%a0,%d0.w),0x600010
      addq.w #2,%d0
      cmp.w #12,%d0
      bne no_wrap
      clr.w %d0
no_wrap:
      move.w %d0,gp_phase
      movem.l (%sp)+,%d0/%a0
      rte
			
gp_phase:
      dc.w 0
gp_control:
      dc.l 0,0,0
    ");

// Main Function
int _main()
{
  unsigned *gp0, *gp1, *gp2;  // Pointers to the bitplanes
  unsigned long temp_addr;
  void *old_autoint5;
  int i;
//  int key;
//  unsigned char counter = 0x6B;

  OSContrastDn();  // Decrease contrast
  OSContrastDn();

  poke(0x600005,0x1);   // Wait for fixed timer to ocure => stop it from coinciding with Autoint 5
	
  //Set LCD data
  poke(0x600012,0x36);  // LCD widh = (64-n)*16 = 160
  poke(0x600013,0x9c);  // LCD height = 256 - n = 100
  poke(0x60001c,0x29);  // LCD row change freq = OSC/((16-n)*8) < n = bits 5..2 > = OSC/48
	
  // Set Interrupt frequency
  poke(0x600015,0xb);   // increase counter freq
  poke(0x600017,0x6b);  // and count from n to ff before trowing autoint5
	
  // Allocate three bitplanes
  temp_addr = (unsigned long) HeapAllocPtr( 3 * 2000 + 8 );  // Room for three bitplanes + filler
  gp0 = (unsigned *) ((temp_addr & ~7ul) + 8);  // Get address aligned on 8 bytes...
  gp1 = gp0 + 1000;  // 1000 word = 2000 byte
  gp2 = gp0 + 2000;

  //Fill the update table with values for the LCD-base register
  gp_control[0] = gp_control[2] = gp_control[4] = (temp_addr>>3) + 1;
  gp_control[1] = gp_control[3] = gp_control[0] + 250;  // 250*8 = 2000
  gp_control[5] = gp_control[1] + 250;
	
  // Start grayscale
  old_autoint5 = (void *)peek_l(0x74);  // Save old handler
  poke(0x600001,0);
  poke_l(0x74,(unsigned long)&grayman);  // Install custom handler
  poke(0x600001,4);
	

  // Draw the stripes...

  // First clear all planes	
  for(i=0; i<3000; i++) gp0[i]=0x0000;
  // Draw first plane
  for(i=4*140+10; i<(7*140+20); i++) gp0[i]=0xffff;
  // Draw second plane
  for(i=2*140+10; i<(4*140+10); i++) gp1[i]=0xffff;
  for(i=5*140+10; i<(7*140+20); i++) gp1[i]=0xffff;
  // Draw third plane
  for(i=1*140+10; i<(2*140+10); i++) gp2[i]=0xffff;
  for(i=3*140+10; i<(5*140+10); i++) gp2[i]=0xffff;
  for(i=6*140+10; i<(7*140+20); i++) gp2[i]=0xffff;
	

/*
  // Allow dynamic changing of counter value for testing
  do {
    key = ngetchx();  // Get key
    if(key == KEY_UP) counter++;
    if(key == KEY_DOWN) counter--;
    poke(0x600017,counter);  // update counter value
  } while(key != 264);  // Exit if ESC pressed
*/

  ngetchx();	// Wait for key press
	
	
  HeapFreePtr((void *)temp_addr);  // Free allocated memmory

  //Turn off gray
  poke(0x600001,0);
  poke_l(0x74,(unsigned long)old_autoint5);  //put old handler back
  poke(0x600001,4);

  // Restore Counter defaults
  poke(0x600015,0x1b);     // restore counter freq
  poke(0x600017,0xb2);     // ...and count 

  // Restore LCD defaults
  poke(0x600012,0x31);     // LCD-widh = 240
  poke(0x600013,0x80);     // LCD-height = 128
  poke(0x60001c,0x21);     // RowChangeFreq = OSC/64
  
  poke_w(0x600010,0x980);  // Restore LCD-base
	
	OSContrastUp();  // Increase contrast
	OSContrastUp();
	
	
  return 0;
} //_main
